typedef ApiOnRequest<R, C> = Future<R> Function(C client);
typedef ApiOnSuccessListener<T> = void Function(T data);
typedef ApiOnFailListener<F> = void Function(F err);

/// [T] 数据类型
/// [R] 数据基类类型 如Result<T>
/// [C] Retrofit Client类型
/// [F] 请求失败时，返回值类型
///
/// example
/// ``` dart
/// class AService<T> extends WingsApiService<T, ResultL<T>, Client, String> {
//   AService.init(ApiOnRequest<Result<T>, Client> onRequest) : super.init(onRequest);
// }
//
// class BService<T> extends WingsApiService<List<T>, ResultL<T>, Client, String> {
//   BService.init(ApiOnRequest<ResultL<T>, Client> onRequest) : super.init(onRequest);
// }
/// ```
///
abstract class WingsApiService<T, R, C, F> {
  ApiOnSuccessListener<T> onSuccessListener;
  ApiOnFailListener<F> onFailListener;
  ApiOnRequest<R, C> onRequest;

  WingsApiService.init(this.onRequest);

  WingsApiService<T, R, C, F> setListener(
    ApiOnSuccessListener<T> onSuccessListener,
    ApiOnFailListener<F> onFailListener,
  ) {
    this.onSuccessListener = onSuccessListener;
    this.onFailListener = onFailListener;
    return this;
  }
}
