import 'package:fish_redux/fish_redux.dart';
import 'package:flutter/material.dart' hide Action, Page;
import 'package:wings/wings.dart';

typedef OnChangeRoute = void Function(
    NavigatorState navigator, String route, dynamic args);
typedef GetPageArgs<T extends WingsPageStateGroup> = dynamic Function(
    T pageStateGroup);

abstract class WingsPageStateGroup {}

abstract class WingsRoutes<T extends WingsPageStateGroup>
    implements AbstractRoutes {
  PageRoutes _pageRoutes;

  T pageStateGroup;

  WingsPageState indexPageState;

  AnimationType anim;

  String routeArgsTag;

  Map<String, WingsPage> pages;

  /// initVisitor
  initVisitor(String path, Page<Object, dynamic> page);

  WingsRoutes(
    this.pageStateGroup, {
    this.anim,
    this.pages,
    this.indexPageState,
    this.routeArgsTag = "wings_args",
  }) {
    _pageRoutes = PageRoutes(
        pages: initPages(),
        visitor: (path, page) {
          initVisitor(path, page);
        });
  }

  get indexPage => buildPage(
      indexPageState.route ??
          (throw FlutterError("WingsRoutes.indexPageState is null.")),
      null);

  Map<String, Page<Object, dynamic>> initPages() {
    return pages
        .map((k, v) => MapEntry(k, v.page))
        .cast<String, Page<Object, dynamic>>();
  }

  MaterialPageRoute onGenerateRoute(RouteSettings settings) {
    return PageRouteTransition(
      animationType: pages[settings.name].anim ?? anim,
      builder: (context) => buildPage(settings.name, settings.arguments),
    );
  }

  @override
  Widget buildPage(String path, arguments) {
    return _pageRoutes.buildPage(path, arguments);
  }

  void change(BuildContext ctx, GetPageArgs<T> getPageArgs,
      OnChangeRoute onChangeRoute) {
    if (getPageArgs != null && onChangeRoute != null) {
      WingsPageState pageState = getPageArgs(pageStateGroup);
      onChangeRoute(
          Navigator.of(ctx), pageState?.route, {routeArgsTag: pageState});
    }
  }

  T getArgs<T>(Map<String, dynamic> args) {
    if (args != null && args[routeArgsTag] != null && args[routeArgsTag] is T) {
      return args[routeArgsTag];
    }
    return null;
  }
}
